package beans;

import java.io.Serializable;

/**
 * Item store in shopping cart
 * 
 * @author Antonin Neumann
 *
 */
public class CartItemBean implements Serializable{

	private static final long serialVersionUID = 919576318023637112L;

	/**
	 * ID item
	 */
	private int idItem;
	
	/**
	 * Name of item
	 */
	private String itemName;
	
	/**
	 * Number of pieces in cart
	 */
	private int pieces;
	
	/**
	 * Price per piece
	 */
	private int pricePerPiece;
	
	public CartItemBean(){}
	
	public CartItemBean(int id, String name, int pieces, int pricePerPiece){
		this.idItem = id;
		this.itemName = name;
		this.pieces = pieces;
		this.pricePerPiece = pricePerPiece;
	}

	public int getIdItem() {
		return idItem;
	}

	public void setIdItem(int idItem) {
		this.idItem = idItem > 0 ? idItem : 0;
	}

	public int getPieces() {
		return pieces;
	}

	public void setPieces(int pieces) {
		this.pieces = pieces > 0 ? pieces : 0;
	}
	
	public void addPieces(int increment){
		setPieces(this.pieces + increment);
	}
	
	
	public int getPricePerPiece() {
		return pricePerPiece;
	}

	public void setPricePerPiece(int pricePerPiece) {
		this.pricePerPiece = pricePerPiece > 0 ? pricePerPiece : 0;
	}
	
	public String getItemName() {
		return itemName;
	}

	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	public int getTotalPrice(){
		return pieces * pricePerPiece;
	}

	@Override
	public String toString() {
		return "CartItemBean [ID=" + idItem + ", itemName=" + itemName
				+ ", pieces=" + pieces + ", pricePerPiece=" + pricePerPiece
				+ ", TOTAL=" + getTotalPrice() + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + idItem;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CartItemBean other = (CartItemBean) obj;
		if (idItem != other.idItem)
			return false;
		return true;
	}

}
