package beans;

import java.text.MessageFormat;
import exception.PortOutOfRange;

/**
 * @author Antonin Neumann
 *
 */
public class DatabaseMysqlBean {
	
	/**
	 * Database encoding
	 */
	private static final String DB_ENCODING = "UTF-8";
	/**
	 * URI to MySQL server
	 */
	private String server;
	/**
	 * Server listening port
	 */
	private int port;
	/**
	 * User name to access to database
	 */
	private String user;
	/**
	 * Password for the user name
	 */
	private String password;
	/**
	 * Name of database
	 */
	private String database;
	
	public DatabaseMysqlBean(String srv, String dbname, String usr) {
		this(srv, dbname, usr, null);
	}
	
	public DatabaseMysqlBean(String srv, String dbname, String usr, String pwd){
		this(srv, dbname, usr, pwd, 0);
	}
	
	public DatabaseMysqlBean(String srv, String dbname, String usr, String pwd, int port){
		this.setServer(srv);
		this.setDatabase(dbname);
		this.setUser(usr);
		this.setPassword(pwd);
		this.setPort(port);
	}
	
	public String getConnectionUri() {
		//return MessageFormat.format("jdbc:mysql://{0}:{1,number,#}/{2}?characterEncoding={3}", this.server, this.port, this.database, DB_ENCODING);
		return MessageFormat.format("jdbc:mysql://{0}/{1}?characterEncoding={2}", this.server, this.database, DB_ENCODING);
	}

	public String getServer() {
		return this.server;
	}

	public void setServer(String server) {
		this.server = server;
	}

	public int getPort() {
		return this.port;
	}

	public void setPort(int port) {
		if(port >= 0 && port < 65535){
			this.port = port;
		}else {
			new PortOutOfRange("Port must be a number between 0 and 65535.");
		}
	}

	public String getUser() {
		return this.user;
	}

	public void setUser(String user) {
		this.user = user;
	}

	public String getPassword() {
		return this.password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getDatabase() {
		return this.database;
	}

	public void setDatabase(String database) {
		this.database = database;
	}
}
