package beans;

import java.io.InputStream;
import java.io.Serializable;

public class ItemBean implements Serializable {

	private static final long serialVersionUID = 6150692790722829789L;
	/**
	 * ID item (PK)
	 */
	private int idItem;
	/**
	 * Name of product
	 */
	private String name;
	/**
	 * Short description of the product
	 */
	private String shortDesc;
	/**
	 * Full description of the product
	 */
	private String fullDesc;
	/**
	 * Price - default currency is CZK
	 */
	private int price;
	/**
	 * Quantity in stock
	 */
	private int quantity;
	/**
	 * Specific features distinguishing the same products
	 */
	private String specificFeatures;
	/**
	 * Product category
	 */
	private String category;
	
	/**
	 * Only for create record in database
	 */
	private InputStream photo = null;
	
	public ItemBean(){
		this(0, null, null, null, 0, 0, null, null);
	}
	
	public ItemBean(int idItem, String name, String shortDesc, String fullDesc) {
		this(idItem, name, shortDesc, fullDesc, 0, 0, null, null);
	}

	public ItemBean(int idItem, String name, String shortDesc, String fullDesc,
			int price, int quantity, String specificFeatures, String category) {
		this.idItem = idItem;
		this.name = name;
		this.shortDesc = shortDesc;
		this.fullDesc = fullDesc;
		this.price = price;
		this.quantity = quantity;
		this.specificFeatures = specificFeatures;
		this.category = category;
	}

	public int getPrice() {
		return price;
	}

	public void setPrice(int price) {
		this.price = price > 0 ? price : 0;
	}

	/**
	 * Alias for {@link #getQuantity()}
	 * @return quantity of item in stock
	 */
	public int getStock(){
		return getQuantity();
	}
	
	public int getQuantity() {
		return quantity;
	}

	public void setQuantity(int quantity) {
		this.quantity = quantity > 0 ? quantity : 0;
	}

	public String getSpecificFeatures() {
		return specificFeatures;
	}

	public void setSpecificFeatures(String specificFeatures) {
		this.specificFeatures = specificFeatures;
	}

	public int getIdItem() {
		return idItem;
	}

	public void setIdItem(int idItem) {
		this.idItem = idItem;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getShortDesc() {
		return shortDesc;
	}

	public void setShortDesc(String shortDesc) {
		this.shortDesc = shortDesc;
	}

	public String getFullDesc() {
		return fullDesc;
	}

	public void setFullDesc(String fullDesc) {
		this.fullDesc = fullDesc;
	}
	
	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	public InputStream getPhoto() {
		return photo;
	}

	public void setPhoto(InputStream photo) {
		this.photo = photo;
	}

	@Override
	public String toString() {
		return "ItemBean [idItem=" + idItem + ", name=" + name + ", price="
				+ price + ", quantity=" + quantity + ", category=" + category
				+ "]";
	}
	
}
