package beans;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Calendar;

import libs.StringUtil;

public class OrderBean implements Serializable {

	private static final long serialVersionUID = 2503554688590777545L;

	/**
	 * ID order (PK)
	 */
	private int idOrder;
	/**
	 * User (FK)
	 */
	private UserBean user;
	/**
	 * Serialized ordered items
	 */
	private String[] orderedItems;
	/**
	 * Serialized billing address
	 */
	private String[] billingAddress;
	/**
	 * Serialized shipping address
	 */
	private String[] shippingAddress;
	/**
	 * Date of order
	 */
	private Calendar orderDate;
	/**
	 * Status Order Processing
	 */
	private String status;
	/**
	 * Price total for all goods
	 */
	private int totalPrice;
	/**
	 * Discount for this order
	 */
	private int discount;
	/**
	 * Total price after discount
	 */
	private int finalPrice;
	
	public OrderBean(){}
	
	public OrderBean(int id, UserBean user, String[] ordItems, String[] billingAddr,
			Calendar orderDate, String status, int totalPrice){
		this(id, user, ordItems, billingAddr, orderDate, status, totalPrice, 0, totalPrice);
	}
	
	public OrderBean(int id, UserBean user, String[] ordItems, String[] billingAddr,
			Calendar orderDate, String status, int totalPrice, int discount, int finalPrice){
		this(id, user, ordItems, billingAddr, billingAddr, orderDate, status, totalPrice, discount, finalPrice);
	}
	
	public OrderBean(int id, UserBean user, String[] ordItems, String[] billingAddr, String[] shippingAddr,
			Calendar orderDate, String status, int totalPrice, int discount, int finalPrice){
		this.setIdOrder(id);
		this.setUser(user);
		this.setOrderedItems(ordItems);
		this.setBillingAddress(billingAddr);
		this.setShippingAddress(shippingAddr);
		this.setOrderDate(orderDate);
		this.setStatus(status);
		this.setTotalPrice(totalPrice);
		this.setDiscount(discount);
		this.setFinalPrice(finalPrice);
	}
	
	
	public int getIdOrder() {
		return idOrder;
	}
	public void setIdOrder(int idOrder) {
		this.idOrder = idOrder;
	}
	public UserBean getUser() {
		return user;
	}
	public void setUser(UserBean user) {
		this.user = user;
	}
	public String[] getOrderedItems() {
		return orderedItems;
	}
	public void setOrderedItems(String[] orderedItems) {
		this.orderedItems = orderedItems;
	}
	public String[] getBillingAddress() {
		return billingAddress;
	}
	public void setBillingAddress(String[] billingAddress) {
		this.billingAddress = billingAddress;
	}
	public void addBillingAddress(String partOfBillingAddress){
		String[] str = null;
		if(getBillingAddress() == null){
			str = new String [] {partOfBillingAddress};
		}else {
			str = StringUtil.concat(getBillingAddress(), partOfBillingAddress);
		}
		setBillingAddress(str);
	}
	public String[] getShippingAddress() {
		return shippingAddress;
	}
	public void setShippingAddress(String[] shippingAddress) {
		this.shippingAddress = shippingAddress;
	}
	public void addShippingAddress(String partOfShippingAddress){
		String[] str = null;
		if(getShippingAddress() == null){
			str = new String [] {partOfShippingAddress};
		}else {
			str = StringUtil.concat(getShippingAddress(), partOfShippingAddress);
		}
		setShippingAddress(str);
	}
	public Calendar getOrderDate() {
		return orderDate;
	}
	public void setOrderDate(Calendar orderDate) {
		this.orderDate = orderDate;
	}
	public String getStatus() {
		return status;
	}
	public void setStatus(String status) {
		this.status = status;
	}
	public int getTotalPrice() {
		return totalPrice;
	}
	public void setTotalPrice(int totalPrice) {
		this.totalPrice = totalPrice;
	}
	public int getDiscount() {
		return discount;
	}
	public void setDiscount(int discount) {
		this.discount = discount;
	}
	public int getFinalPrice() {
		return finalPrice;
	}
	public void setFinalPrice(int finalPrice) {
		this.finalPrice = finalPrice;
	}

	@Override
	public String toString() {
		String userId = this.user == null ? "no user" : String.valueOf(user.getIdUser());
		return "OrderBean [idOrder=" + idOrder + ", user=" + userId
				+ ", orderedItems=" + Arrays.toString(orderedItems)
				+ ", billingAddress=" + Arrays.toString(billingAddress)
				+ ", shippingAddress=" + Arrays.toString(shippingAddress)
				+ ", orderDate=" + orderDate.getTimeInMillis() + ", status=" + status
				+ ", totalPrice=" + totalPrice + ", discount=" + discount
				+ ", finalPrice=" + finalPrice + "]";
	}
	
}
