package beans;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;

/**
 * @author Antonin Neumann
 *
 */
public class UserBean implements Serializable{
	
	private static final long serialVersionUID = -423393176977103732L;
	/**
	 * ID user (PK)
	 */
	private int idUser;
	/**
	 * User login name
	 */
	private String login;
	/**
	 * E-mail address
	 */
	private String email;
	/**
	 * Password hash
	 */
    private String password;
    /**
     * Name of user
     */
    private String name;
    /**
     * Lastname of user
     */
    private String lastname;
    /**
     * Access to administration e-shop
     */
    private boolean isAdmin;
    /**
     * Date of birth
     */
    private Calendar dateOfBirth;
    /**
     * Verified user account (e-mail)
     */
    private boolean isValid;
    /**
     * Percentage discount
     */
    private int discount;
    /**
     * Address list of user<br>
     * (Can be <strong>null</strong>)
     */
    private List<AddressBean> addresses;
    
    /**
     * Only for create new user in sign up
     */
    public UserBean(){}
    
    public UserBean(int id, String login, String pass) {
    	this(id, login, pass, null, null, null, false, null, false);
    }
    
    public UserBean(int id, String login, String pass, String email, String name, String lastname, 
    		boolean isAdmin, Calendar dateOfBirth, boolean isValid){
    	this(id, login, pass, email, name, lastname, isAdmin, dateOfBirth, isValid, 0);
    }
    
    public List<AddressBean> getAddresses() {
		return addresses;
	}

	public void setAddresses(List<AddressBean> addresses) {
		this.addresses = addresses;
	}

	public UserBean(int id, String login, String pass, String email, String name, String lastname, 
    		boolean isAdmin, Calendar dateOfBirth, boolean isValid, int discount){
    	this.setIdUser(id);
    	this.setLogin(login);
    	this.setPassword(pass);
    	this.setEmail(email);
    	this.setName(name);
    	this.setLastname(lastname);
    	this.setAdmin(isAdmin);
    	this.setDateOfBirth(dateOfBirth);
    	this.setValid(isValid);
    	this.setDiscount(discount);
    }

	public int getIdUser() {
		return idUser;
	}

	public void setIdUser(int idUser) {
		this.idUser = idUser;
	}

	public String getLogin() {
		return login;
	}

	public void setLogin(String login) {
		this.login = login;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getLastname() {
		return lastname;
	}

	public void setLastname(String lastname) {
		this.lastname = lastname;
	}

	public boolean isAdmin() {
		return isAdmin;
	}

	public void setAdmin(boolean isAdmin) {
		this.isAdmin = isAdmin;
	}

	public Calendar getDateOfBirth() {
		return dateOfBirth;
	}

	public void setDateOfBirth(Calendar dateOfBirth) {
		this.dateOfBirth = dateOfBirth;
	}
	
	public int getAge(){
		return Calendar.getInstance().get(Calendar.YEAR) - getDateOfBirth().get(Calendar.YEAR);
	}

	public boolean isValid() {
		return isValid;
	}

	public void setValid(boolean isValid) {
		this.isValid = isValid;
	}

	public int getDiscount() {
		return discount;
	}

	public void setDiscount(int discount) {
		this.discount = discount;
	}
	
	public boolean hasDiscount(){
		if(this.discount > 0)
			return true;
		else 
			return false;
	}

	@Override
	public String toString() {
		SimpleDateFormat sdf =  new SimpleDateFormat("dd. MM. yyyy");
		return "UserBean [idUser=" + idUser + ", login=" + login + ", email="
				+ email + ", password=" + password + ", name=" + name
				+ ", lastname=" + lastname + ", isAdmin=" + isAdmin
				+ ", dateOfBirth=" + sdf.format(dateOfBirth.getTime()) + ", isValid=" + isValid
				+ ", discount=" + discount + "]";
	}  
}
