package dao;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import beans.DatabaseMysqlBean;

public abstract class AMysqlDao {

	/**
	 * Bean of database MySQL connection
	 */
	private DatabaseMysqlBean dbConnection;
	/**
	 * Properties object
	 * @see Properties
	 */
	private Properties prop = new Properties();
	/**
	 * FileInputStream object
	 * @see FileInputStream
	 */
	private InputStream input = null;
	/**
	 * Is database on local server (development)
	 */
	private boolean isLocal;

	public AMysqlDao() throws ClassNotFoundException {
		this(false);
	}
	
	public AMysqlDao(boolean local) throws ClassNotFoundException{
		this.isLocal = local;
		
		Class.forName("com.mysql.jdbc.Driver");
			
		try {
			ClassLoader classloader = Thread.currentThread().getContextClassLoader();
			input = classloader.getResourceAsStream("config/db.properties");
			prop.load(input);
			setupConnection();
		} catch (FileNotFoundException e) {
			System.err.println("Property file with data access to databases not found!");
			e.printStackTrace();
			System.err.println("Trying reserve setup for connection...");
			reserveSetupConnection();
		} catch (IOException e) {
			System.err.println("Cannot read property file!");
			e.printStackTrace();
		}
	}
	
	/**
	 * Method returns Connection to database of SQL type
	 * 
	 * @return Connection
	 * @throws SQLException
	 */
	protected Connection getConnection() throws SQLException {
		return DriverManager.getConnection(
			this.dbConnection.getConnectionUri(), 
			this.dbConnection.getUser(), 
			this.dbConnection.getPassword()
		);
	}
	
	protected void setupConnection(){
		if(this.isLocal){
			this.dbConnection = new DatabaseMysqlBean(
				prop.getProperty("locServer"), 
				prop.getProperty("locDatabase"), 
				prop.getProperty("locUser"), 
				prop.getProperty("locPassword"),
				Integer.parseInt(prop.getProperty("locPort"))
			);
		}else {
			this.dbConnection = new DatabaseMysqlBean(
				prop.getProperty("server"), 
				prop.getProperty("database"), 
				prop.getProperty("user"), 
				prop.getProperty("password"),
				Integer.parseInt(prop.getProperty("port"))
			);
		}
	}
	
	protected void reserveSetupConnection(){
		if(this.isLocal){
			this.dbConnection = new DatabaseMysqlBean("localhost", "pia_neumann", "root", "");
		} else {
			this.dbConnection = new DatabaseMysqlBean("localhost", "neumann", "pia", "pia", 3306);
		}
	}

}
