package dao;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.dbutils.DbUtils;

import beans.AddressBean;
import beans.UserBean;
import exception.CustomException;

public class AddressDao extends AMysqlDao implements IAddressDao {
	
	private Connection connection = null;
	private PreparedStatement preparedStatement = null;
	private ResultSet resultSet = null;

	public AddressDao() throws ClassNotFoundException {
		super();
	}

	public AddressBean create(AddressBean instance) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("INSERT INTO address (id_user, street, number, town, state, zip_code) "
					+ "VALUES (?,?,?,?,?,?)");

			preparedStatement.setInt(1, instance.getUser().getIdUser());
			preparedStatement.setString(2, instance.getStreet());
			preparedStatement.setInt(3, instance.getNumber());
			preparedStatement.setString(4, instance.getTown());
			preparedStatement.setString(5, instance.getState());
			preparedStatement.setInt(6, instance.getZipCode());

			preparedStatement.executeUpdate();
			return read(instance.getIdAddress());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public AddressBean read(Integer instance) throws SQLException {
		UserDao userAccess = null;
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("SELECT * FROM address WHERE id_address='"+instance+"';");
			resultSet = preparedStatement.executeQuery();

			userAccess = new UserDao();

			
			if(resultSet.next()) {
				AddressBean address = new AddressBean(
						resultSet.getInt("id_address"),
						userAccess.read(resultSet.getInt("id_user")), 
						resultSet.getString("street"),
						resultSet.getInt("number"), 
						resultSet.getString("town"), 
						resultSet.getString("state"), 
						resultSet.getInt("zip_code"));
				return address;
			}

		} catch (SQLException e) {
			throw e;
		} catch (ClassNotFoundException e) {
			CustomException.printException(e, "Cannot find UserDao class!");
		} catch (IOException e) {
			 CustomException.printFullException(e, "IOException");
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}
	
	public AddressBean update(AddressBean instance) throws SQLException {
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("UPDATE address SET street = ?, number = ?, town = ?,"
					+ "state = ?, zip_code = ? WHERE id_address = ?");

			preparedStatement.setString(1, instance.getStreet());
			preparedStatement.setInt(2, instance.getNumber());
			preparedStatement.setString(3, instance.getTown());
			preparedStatement.setString(4, instance.getState());
			preparedStatement.setInt(5, instance.getZipCode());

			preparedStatement.setInt(6, instance.getIdAddress());

			preparedStatement.executeUpdate();
			return read(instance.getIdAddress());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public boolean delete(AddressBean instance) throws SQLException {
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("DELETE FROM address WHERE id_address = ?");
			preparedStatement.setInt(1, instance.getIdAddress());

			if(preparedStatement.executeUpdate() != 0){
				return true;
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return false;
	}
	
	public boolean delete(UserBean instance) throws SQLException {
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("DELETE FROM address WHERE id_user = ?");
			preparedStatement.setInt(1, instance.getIdUser());

			if(preparedStatement.executeUpdate() != 0){
				return true;
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return false;
	}

	public List<AddressBean> getAddresses() throws SQLException {
		List<AddressBean> addresses = new ArrayList<AddressBean>();
		UserDao userAccess = null;

		try {
			connection = this.getConnection();
			preparedStatement = connection.prepareStatement("SELECT * FROM address;");
			resultSet = preparedStatement.executeQuery();

			userAccess = new UserDao();
			
			while (resultSet.next()) {
				AddressBean adr = new AddressBean(
						resultSet.getInt("id_address"),
						userAccess.read(resultSet.getInt("id_user")), 
						resultSet.getString("street"),
						resultSet.getInt("number"), 
						resultSet.getString("town"), 
						resultSet.getString("state"), 
						resultSet.getInt("zip_code")
						);
				addresses.add(adr);
			}

		} catch (SQLException e) {
			throw e;
		} catch (ClassNotFoundException e) {
			CustomException.printException(e, "Cannot find UserDao class!");
		} catch (IOException e) {
			CustomException.printFullException(e, "IOException");
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return addresses;
	}

	public AddressBean getAddress(UserBean user) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("SELECT * FROM address WHERE id_user='"+user.getIdUser()+"';");
			preparedStatement.setMaxRows(1);
			resultSet = preparedStatement.executeQuery();

			if(resultSet.next()) {
				AddressBean address = new AddressBean(
						resultSet.getInt("id_address"), 
						resultSet.getString("street"), 
						resultSet.getInt("number"), 
						resultSet.getString("town"), 
						resultSet.getString("state"),  
						resultSet.getInt("zip_code")
					);
				return address;
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}

	
}
