package dao;

import java.io.IOException;
import java.sql.SQLException;

import org.apache.commons.codec.DecoderException;

/**
 * CRUD generic DAO interface
 * 
 * @author Antonin Neumann
 *
 * @param <E> Bean object
 * @param <I> Primary key
 */
public interface IGenericDao<E, I> {
	/**
	 * Create new record from object instance and save into database
	 * 
	 * @param instance Bean object
	 * @return Bean object
	 * @throws SQLException
	 * @throws IOException 
	 * @throws DecoderException 
	 * @throws ClassNotFoundException 
	 */
	E create(E instance) throws SQLException, IOException, ClassNotFoundException, DecoderException;
	
	/**
	 * Loading record from database to Bean object
	 * 
	 * @param instance (PK)
	 * @return Bean object
	 * @throws SQLException
	 * @throws DecoderException 
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 */
	E read(I instance) throws SQLException, ClassNotFoundException, IOException, DecoderException;
	
	/**
	 * Update record
	 * 
	 * @param instance Bean object
	 * @return Bean object 
	 * @throws SQLException 
	 */
	E update(E instance) throws SQLException;
	
	/**
	 * Delete record from database
	 * 
	 * @param instance Bean object
	 * @return TRUE if object was deleted, FALSE otherwise
	 * @throws SQLException 
	 */
	boolean delete (E instance) throws SQLException;
}
