package dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.dbutils.DbUtils;

import beans.CategoryBean;
import beans.ItemBean;

public class ItemDao extends AMysqlDao implements IItemDao {

	private Connection connection = null;
	private PreparedStatement preparedStatement = null;
	private ResultSet resultSet = null;
	
	public ItemDao() throws ClassNotFoundException {
		super();
	}

	public ItemBean create(ItemBean instance) throws SQLException {
		int category = getIdCategory(instance.getCategory());
		boolean photo = instance.getPhoto() == null ? false : true;
		try {
			connection = this.getConnection();

			if(photo){
				preparedStatement = connection.prepareStatement("INSERT INTO item (name, short_desc, full_desc, price, quantity, specific_features, id_category, photo) "
						+ "VALUES (?,?,?,?,?,?,?,?)");
			} else {
				preparedStatement = connection.prepareStatement("INSERT INTO item (name, short_desc, full_desc, price, quantity, specific_features, id_category) "
						+ "VALUES (?,?,?,?,?,?,?)");
			}
			
			preparedStatement.setString(1, instance.getName());
			preparedStatement.setString(2, instance.getShortDesc());
			preparedStatement.setString(3, instance.getFullDesc());
			preparedStatement.setInt(4, instance.getPrice());
			preparedStatement.setInt(5, instance.getQuantity());
			preparedStatement.setString(6, instance.getSpecificFeatures());
			preparedStatement.setInt(7, category);
			if(photo){
				preparedStatement.setBlob(8, instance.getPhoto());
			}

			preparedStatement.executeUpdate();
			return read(instance.getIdItem());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public ItemBean read(Integer instance) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT *, category.name AS category FROM item"
					+ " LEFT JOIN category ON item.id_category = category.id_category WHERE id_item = ?");
			preparedStatement.setInt(1, instance);
			resultSet = preparedStatement.executeQuery();

			if(resultSet.next()) {
				ItemBean item = new ItemBean(
						resultSet.getInt("id_item"), 
						resultSet.getString("name"), 
						resultSet.getString("short_desc"), 
						resultSet.getString("full_desc"), 
						resultSet.getInt("price"), 
						resultSet.getInt("quantity"),
						resultSet.getString("specific_features"),
						resultSet.getString("category")
					);
				return item;
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}

	public ItemBean update(ItemBean instance) throws SQLException {
		int category = getIdCategory(instance.getCategory());
		boolean photo = instance.getPhoto() == null ? false : true;
		try {
			connection = this.getConnection();
				
			if(photo){
				preparedStatement = connection.prepareStatement("UPDATE item SET name = ?, short_desc = ?, full_desc = ?,"
						+ " price = ?, quantity = ?, specific_features = ?, id_category = ?, photo = ? WHERE id_item = ?");
			} else {
				preparedStatement = connection.prepareStatement("UPDATE item SET name = ?, short_desc = ?, full_desc = ?,"
					+ " price = ?, quantity = ?, specific_features = ?, id_category = ? WHERE id_item = ?");
			}

			preparedStatement.setString(1, instance.getName());
			preparedStatement.setString(2, instance.getShortDesc());
			preparedStatement.setString(3, instance.getFullDesc());
			preparedStatement.setInt(4, instance.getPrice());
			preparedStatement.setInt(5, instance.getQuantity());
			preparedStatement.setString(6, instance.getSpecificFeatures());
			preparedStatement.setInt(7, category);
			if(photo){
				preparedStatement.setBlob(8, instance.getPhoto());
				preparedStatement.setInt(9, instance.getIdItem());
			}else {
				preparedStatement.setInt(8, instance.getIdItem());
			}

			preparedStatement.executeUpdate();
			return read(instance.getIdItem());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public boolean delete(ItemBean instance) throws SQLException {
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("DELETE FROM item WHERE id_item = ?");
			preparedStatement.setInt(1, instance.getIdItem());

			if(preparedStatement.executeUpdate() != 0){
				return true;
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return false;
	}

	public List<ItemBean> getItems() throws SQLException {
		List<ItemBean> items = new ArrayList<ItemBean>();
		ItemBean item;
		
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT *, category.name AS category FROM item"
					+ " LEFT JOIN category ON item.id_category = category.id_category");
			resultSet = preparedStatement.executeQuery();

			while (resultSet.next()) {
				item = new ItemBean(
					resultSet.getInt("id_item"), 
					resultSet.getString("name"), 
					resultSet.getString("short_desc"), 
					resultSet.getString("full_desc"), 
					resultSet.getInt("price"), 
					resultSet.getInt("quantity"),
					resultSet.getString("specific_features"),
					resultSet.getString("category")
				);
				items.add(item);
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return items;
	}

	public List<ItemBean> getHotItems(int limit) throws SQLException {
		List<ItemBean> items = new ArrayList<ItemBean>();
		ItemBean item;
		
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT *, category.name AS category FROM item"
					+ " LEFT JOIN category ON item.id_category = category.id_category ORDER BY quantity DESC");
			if(limit > 0){	
				preparedStatement.setMaxRows(limit);
			}
			resultSet = preparedStatement.executeQuery();

			while (resultSet.next()) {
				item = new ItemBean(
					resultSet.getInt("id_item"), 
					resultSet.getString("name"), 
					resultSet.getString("short_desc"), 
					resultSet.getString("full_desc"), 
					resultSet.getInt("price"), 
					resultSet.getInt("quantity"),
					resultSet.getString("specific_features"),
					resultSet.getString("category")
				);
				items.add(item);
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return items;
	}
	
	public List<ItemBean> getItemsByCategory(String name) throws SQLException {
		return getItemsByCategory(name, 0);
	}
	
	public List<ItemBean> getItemsByCategory(String name, int limit) throws SQLException {
		List<ItemBean> items = new ArrayList<ItemBean>();
				
		try {
			connection = this.getConnection();

			
			preparedStatement = connection.prepareStatement(
					"SELECT *, category.name AS category FROM item "
					+ "LEFT JOIN category ON item.id_category = category.id_category WHERE category.name = ?");
			if(limit > 0){	
				preparedStatement.setMaxRows(limit);
			}
			preparedStatement.setString(1, name);
			resultSet = preparedStatement.executeQuery();

			while (resultSet.next()) {
				ItemBean item = new ItemBean(
						resultSet.getInt("id_item"), 
						resultSet.getString("name"), 
						resultSet.getString("short_desc"), 
						resultSet.getString("full_desc"), 
						resultSet.getInt("price"), 
						resultSet.getInt("quantity"),
						resultSet.getString("specific_features"),
						resultSet.getString("category")
					);
				items.add(item);
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return items;
	}
	
	public List<ItemBean> getItemsByCategory(int idCategory) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT name FROM category WHERE id_category = ?");
			preparedStatement.setInt(1, idCategory);
			preparedStatement.setMaxRows(1);
			resultSet = preparedStatement.executeQuery();
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return getItemsByCategory(resultSet.getString("name"));
	}
	
	public List<ItemBean> getDiscountItems() throws SQLException{
		return getDiscountItems(0);
	}
	
	public List<ItemBean> getDiscountItems(int limit) throws SQLException{
		List<ItemBean> items = new ArrayList<ItemBean>();
		
		try {
			connection = this.getConnection();

			
			preparedStatement = connection.prepareStatement(
					"SELECT *, category.name AS category FROM item "
					+ "LEFT JOIN category ON item.id_category = category.id_category WHERE item.is_discount = 1");
			if(limit > 0){	
				preparedStatement.setMaxRows(limit);
			}
			resultSet = preparedStatement.executeQuery();

			while (resultSet.next()) {
				ItemBean item = new ItemBean(
						resultSet.getInt("id_item"), 
						resultSet.getString("name"), 
						resultSet.getString("short_desc"), 
						resultSet.getString("full_desc"), 
						resultSet.getInt("price"), 
						resultSet.getInt("quantity"),
						resultSet.getString("specific_features"),
						resultSet.getString("category")
					);
				items.add(item);
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return items;
	}

	public byte[] getItemImage(int idItem) throws SQLException{
		try {
			connection = this.getConnection();
	
			preparedStatement = connection.prepareStatement("SELECT photo FROM item WHERE id_item = ?");
			preparedStatement.setInt(1, idItem);
			resultSet = preparedStatement.executeQuery();
			if (resultSet.next()) {
	            return resultSet.getBytes("photo");
	        }
			
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}

	private int getIdCategory(String name) throws SQLException{
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT id_category FROM category WHERE name LIKE ?");
			preparedStatement.setString(1, "%"+name+"%");
			resultSet = preparedStatement.executeQuery();

			if(resultSet.next()) {
				return resultSet.getInt("id_category");
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return 0;
	}

	public List<CategoryBean> getCategories() throws SQLException {
		List<CategoryBean> cat = new ArrayList<CategoryBean>();
		CategoryBean category;
		
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT * FROM category");
			resultSet = preparedStatement.executeQuery();

			while (resultSet.next()) {
				category = new CategoryBean(
						resultSet.getInt("id_category"), 
						resultSet.getString("name"), 
						resultSet.getString("desc")
					);
				cat.add(category);
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return cat;
	}
	
	public CategoryBean getCategory(int id) throws SQLException {
		CategoryBean category = null;
		
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT * FROM category WHERE id_category = ?");
			preparedStatement.setMaxRows(1);
			preparedStatement.setInt(1, id);
			resultSet = preparedStatement.executeQuery();

			if(resultSet.next()) {
				category = new CategoryBean(
						resultSet.getInt("id_category"), 
						resultSet.getString("name"), 
						resultSet.getString("desc")
					);
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return category;
	}

	public boolean deleteCategory(CategoryBean instance) throws SQLException {
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("DELETE FROM category WHERE id_category = ?");
			preparedStatement.setInt(1, instance.getIdCategory());

			if(preparedStatement.executeUpdate() != 0){
				return true;
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return false;
	}

}
