package dao;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import libs.DateConvert;
import libs.StringSerialize;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.dbutils.DbUtils;

import beans.OrderBean;

public class OrderDao  extends AMysqlDao implements IOrderDao {

	private Connection connection = null;
	private PreparedStatement preparedStatement = null;
	private ResultSet resultSet = null;
	
	public OrderDao() throws ClassNotFoundException {
		super();
	}

	public OrderBean create(OrderBean instance) throws SQLException, IOException, ClassNotFoundException, DecoderException {
		try {
			connection = this.getConnection();
			
			preparedStatement = connection.prepareStatement("INSERT INTO `order` "
					+ "(`id_user`, `ordered_items`, `billing_address`, `shipping_address`, `order_date`, "
					+ "`id_status`, `total_price`, `discount`, `final_price`) "
					+ "VALUES (?,?,?,?,?,?,?,?,?)");
			if(instance.getUser() != null){
				preparedStatement.setInt(1, instance.getUser().getIdUser());
			}else {
				preparedStatement.setInt(1, 0);
			}
			preparedStatement.setString(2, StringSerialize.serialize(instance.getOrderedItems()));
			preparedStatement.setString(3, StringSerialize.serialize(instance.getBillingAddress()));
			preparedStatement.setString(4, StringSerialize.serialize(instance.getShippingAddress()));
			preparedStatement.setDate(5, new java.sql.Date(instance.getOrderDate().getTimeInMillis()));
			preparedStatement.setInt(6, 1);
			preparedStatement.setInt(7, instance.getTotalPrice());
			preparedStatement.setInt(8, instance.getDiscount());
			preparedStatement.setInt(9, instance.getFinalPrice());
			
			System.out.println(preparedStatement.toString());
			
			preparedStatement.executeUpdate();
			return read(instance.getIdOrder());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public OrderBean read(Integer instance) throws SQLException, ClassNotFoundException, IOException, DecoderException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT *, order_status.name AS status FROM `order`"
					+ " LEFT JOIN order_status ON `order`.id_status = order_status.id_status WHERE `order`.id_order = ?");
			preparedStatement.setInt(1, instance);
			resultSet = preparedStatement.executeQuery();

			UserDao userAccess = new UserDao();
			
			if(resultSet.next()) {
				OrderBean order = new OrderBean(
						resultSet.getInt("id_order"), 
						userAccess.read(resultSet.getInt("id_user")), 
						StringSerialize.deserialize(resultSet.getString("ordered_items")), 
						StringSerialize.deserialize(resultSet.getString("billing_address")),
						StringSerialize.deserialize(resultSet.getString("shipping_address")), 
						DateConvert.DateToCalendar(resultSet.getDate("order_date")), 
						resultSet.getString("status"), 
						resultSet.getInt("total_price"), 
						resultSet.getInt("discount"),
						resultSet.getInt("final_price")
					);
				return order;
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}

	/**
	 * Orders will do not update.
	 */
	public OrderBean update(OrderBean instance) throws SQLException {
		return null;
	}
	/**
	 * Orders are not removed.
	 */
	public boolean delete(OrderBean instance) throws SQLException {
		return false;
	}
	
	public List<OrderBean> getOrders() throws SQLException, ClassNotFoundException, IOException, DecoderException {
		List<OrderBean> orders = new ArrayList<OrderBean>();
		OrderBean order;
		
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement(
					"SELECT *, order_status.name AS status FROM `order`"
					+ " LEFT JOIN order_status ON `order`.id_status = order_status.id_status ORDER BY `order`.id_order DESC");
			resultSet = preparedStatement.executeQuery();

			UserDao userAccess = new UserDao();
			
			while (resultSet.next()) {
				order = new OrderBean(
						resultSet.getInt("id_order"), 
						userAccess.read(resultSet.getInt("id_user")), 
						StringSerialize.deserialize(resultSet.getString("ordered_items")), 
						StringSerialize.deserialize(resultSet.getString("billing_address")),
						StringSerialize.deserialize(resultSet.getString("shipping_address")), 
						DateConvert.DateToCalendar(resultSet.getDate("order_date")), 
						resultSet.getString("status"), 
						resultSet.getInt("total_price"), 
						resultSet.getInt("discount"),
						resultSet.getInt("final_price")
					);
				orders.add(order);
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return orders;
	}
}
