package dao;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import libs.DateConvert;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.dbutils.DbUtils;

import beans.UserBean;

public class UserDao extends AMysqlDao implements IUserDao {

	private Connection connection = null;
	private PreparedStatement preparedStatement = null;
	private ResultSet resultSet = null;
	
	public UserDao() throws ClassNotFoundException, IOException {
		super();
	}

	public UserBean create(UserBean instance) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("INSERT INTO user (login, pass, email, name, lastname, date_of_birth) "
					+ "VALUES (?,?,?,?,?,?)");

			preparedStatement.setString(1,  instance.getLogin());
			preparedStatement.setString(2, DigestUtils.shaHex((instance.getPassword())));
			preparedStatement.setString(3, instance.getEmail());
			preparedStatement.setString(4, instance.getName());
			preparedStatement.setString(5, instance.getLastname());
			preparedStatement.setDate(6, new java.sql.Date(instance.getDateOfBirth().getTimeInMillis()));

			preparedStatement.executeUpdate();
			return read(instance.getIdUser());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public UserBean read(Integer instance) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("SELECT * FROM user WHERE id_user='"+instance+"';");
			resultSet = preparedStatement.executeQuery();

			if(resultSet.next()) {
				UserBean user = new UserBean(
						resultSet.getInt("id_user"),
						resultSet.getString("login"), 
						resultSet.getString("pass"), 
						resultSet.getString("email"), 
						resultSet.getString("name"), 
						resultSet.getString("lastname"), 
						resultSet.getBoolean("is_admin"), 
						DateConvert.DateToCalendar(resultSet.getDate("date_of_birth")), 
						resultSet.getBoolean("valid"), 
						resultSet.getInt("discount")
						);
				return user;
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}
	
	public UserBean read(String userName) throws SQLException {
		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("SELECT * FROM user WHERE login='" + userName + "';");
			preparedStatement.setMaxRows(1);
			resultSet = preparedStatement.executeQuery();

			if(resultSet.next()) {
				UserBean user = new UserBean(
						resultSet.getInt("id_user"),
						resultSet.getString("login"), 
						resultSet.getString("pass"), 
						resultSet.getString("email"), 
						resultSet.getString("name"), 
						resultSet.getString("lastname"), 
						resultSet.getBoolean("is_admin"), 
						DateConvert.DateToCalendar(resultSet.getDate("date_of_birth")), 
						resultSet.getBoolean("valid"), 
						resultSet.getInt("discount")
						);
				return user;
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return null;
	}

	public UserBean update(UserBean instance) throws SQLException {
		
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("UPDATE user SET email = ?, name = ?,"
					+ "lastname = ?, is_admin = ?, date_of_birth = ?, valid = ?, discount = ? WHERE id_user = ?");


			preparedStatement.setString(1,  instance.getEmail());
			preparedStatement.setString(2, instance.getName());
			preparedStatement.setString(3, instance.getLastname());
			preparedStatement.setBoolean(4, instance.isAdmin());
			preparedStatement.setDate(5, new java.sql.Date(instance.getDateOfBirth().getTimeInMillis()));
			preparedStatement.setBoolean(6, instance.isValid());
			preparedStatement.setInt(7, instance.getDiscount());
			
			preparedStatement.setInt(8, instance.getIdUser());

			preparedStatement.executeUpdate();
			return read(instance.getIdUser());

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
	}

	public boolean delete(UserBean instance) throws SQLException {
		try {
			connection = this.getConnection();
					
			preparedStatement = connection.prepareStatement("DELETE FROM user WHERE id_user = ?");
			preparedStatement.setInt(1, instance.getIdUser());

			if(preparedStatement.executeUpdate() != 0){
				return true;
			}
		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}
		return false;
	}

	public List<UserBean> getUsers() throws SQLException {
		List<UserBean> users = new ArrayList<UserBean>();

		try {
			connection = this.getConnection();

			preparedStatement = connection.prepareStatement("SELECT * FROM user;");
			resultSet = preparedStatement.executeQuery();

			while (resultSet.next()) {
				UserBean user = new UserBean(
						resultSet.getInt("id_user"),
						resultSet.getString("login"), 
						resultSet.getString("pass"), 
						resultSet.getString("email"), 
						resultSet.getString("name"), 
						resultSet.getString("lastname"), 
						resultSet.getBoolean("is_admin"), 
						DateConvert.DateToCalendar(resultSet.getDate("date_of_birth")), 
						resultSet.getBoolean("valid"), 
						resultSet.getInt("discount")
						);
				users.add(user);
			}

		} catch (SQLException e) {
			throw e;
		} finally {
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(preparedStatement);
			DbUtils.closeQuietly(connection);
		}

		return users;
	}
}
