package exception;

import java.sql.SQLException;


/**
 * Custom Exception for printing custom stackTrace
 * 
 * @author Antonin Neumann
 *
 */
public class CustomException extends Exception {
	
	private static final long serialVersionUID = -7903841153351294805L;

	/**
	 * Print exception message a stackTrace on console.<br>
	 * Print only 3 stackTrace.
	 * 
	 * @param e Exception
	 * @param msg Message
	 */
	public static void printException(Exception e, String msg){
		System.err.print("\n" + msg);
		System.err.println(" | type " + e.getClass() + ": " + e.getMessage());
		
		if(e instanceof SQLException){
			System.err.println("SQLState: " + ((SQLException)e).getSQLState() 
					+ ", Error Code: " + ((SQLException)e).getErrorCode());
		}
		
		StackTraceElement[] trace = e.getStackTrace();
		int k = trace.length > 3 ? 3 : trace.length;
		String x;
		for (int i = 0; i < k; i++) {
			if(trace[i].getFileName() != null){
				x = trace[i].getFileName() + ":" + trace[i].getLineNumber();
				System.err.println(" in (" + x + ")");
			}
		}
	}
	
	/**
	 *  Print exception message a stackTrace on console.<br>
	 *  Print full stackTrace.
	 * 
	 * @param e Exception object
	 * @param msg Custom message
	 */
	public static void printFullException(Exception e, String msg){
		System.err.print("\n" + msg);
		System.err.println(" | type " + e.getClass() + ":");
		
		String x;
		for (StackTraceElement trace : e.getStackTrace()) {
			if(trace.getFileName() != null){
				x = trace.getFileName() + ":" + trace.getLineNumber();
				System.err.println(" in (" + x + ")");
			}
		}
	}
}
