package servlet;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import libs.FormChecker;
import libs.UriParser;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.FilenameUtils;

import beans.CategoryBean;
import beans.ItemBean;
import beans.OrderBean;
import dao.ItemDao;
import dao.OrderDao;
import exception.CustomException;

/**
 * Servlet implementation class Admin
 */
public class Admin extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public Admin() {
        super();
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String[] uriSegments = UriParser.getUriSegments(request.getRequestURI().toString());
		int uriSegmentsSize = uriSegments.length;
		String lastUriSegment = uriSegments[uriSegmentsSize-1];
		String pageType = uriSegments[uriSegmentsSize-2];
		
		ItemDao itemAccess = null;
		try {
			itemAccess = new ItemDao();
		} catch (ClassNotFoundException e) {
			CustomException.printFullException(e, "Cannot load list of items!");
		}
		
		if(lastUriSegment.equals("admin") || lastUriSegment.equals("item")){
			response.sendRedirect("/neumann/admin/item/show");
		}else if(lastUriSegment.equals("category")){
			response.sendRedirect("/neumann/admin/item/show#categories");
		}else if(lastUriSegment.equals("order")){
			response.sendRedirect("/neumann/admin/order/show");
		}
		
		if(pageType.equals("category")){
			if(lastUriSegment.equals("delete")){ /* ITEM DELETE */
				CategoryBean cat = null;
				try {
					cat = itemAccess.getCategory(Integer.valueOf(request.getParameter("id")));
					itemAccess.deleteCategory(cat);
				} catch (NumberFormatException e) {
					CustomException.printException(e, "Cannot convert parameter to integer!");
				} catch (SQLException e) {
					CustomException.printException(e, "SQL error!");
					e.printStackTrace();
				}
				response.sendRedirect("/neumann/admin/item/show");
			}
		}else if(pageType.equals("item")){
			if(lastUriSegment.equals("show")){ /* ITEM SHOW */
				List<ItemBean> itemList = null;
				List<CategoryBean> catList = null;
				try {
					itemList = itemAccess.getItems();
					catList = itemAccess.getCategories();
				} catch (SQLException e) {
					CustomException.printFullException(e, "SQL error!");
				}
				request.setAttribute("itemList", itemList);
				request.setAttribute("catList", catList);
				
				request.getRequestDispatcher("/WEB-INF/admin/item-show.jsp").forward(request, response);
			}else if(lastUriSegment.equals("view")){ /* ITEM VIEW */
				ItemBean item = null;
				try {
					item = itemAccess.read(Integer.valueOf(request.getParameter("id")));
				} catch (NumberFormatException e) {
					CustomException.printException(e, "Cannot convert parameter to integer!");
				} catch (SQLException e) {
					CustomException.printException(e, "SQL error!");
					e.printStackTrace();
				}
				request.setAttribute("item", item);
				
				request.getRequestDispatcher("/WEB-INF/admin/item-view.jsp").forward(request, response);
			}else if(lastUriSegment.equals("delete")){ /* ITEM DELETE */
				ItemBean item = null;
				try {
					item = itemAccess.read(Integer.valueOf(request.getParameter("id")));
					itemAccess.delete(item);
				} catch (NumberFormatException e) {
					CustomException.printException(e, "Cannot convert parameter to integer!");
				} catch (SQLException e) {
					CustomException.printException(e, "SQL error!");
					e.printStackTrace();
				}
				response.sendRedirect("/neumann/admin/item/show");
			}else if(lastUriSegment.equals("edit")){ /* ITEM EDIT */
				request.setAttribute("edit", true);
				
				List<CategoryBean> categories = null;
				try {
					categories = itemAccess.getCategories();
				} catch (SQLException e) {
					CustomException.printFullException(e, "SQL error!");
				}
				request.setAttribute("categories", categories);
				
				Map<String, String> data = new HashMap<String,String>();
				ItemBean item = null;
				try {
					item = itemAccess.read(Integer.valueOf(request.getParameter("id")));
					data.put("idItem", Integer.toString(item.getIdItem()));
					data.put("name", item.getName());
					data.put("price", Integer.toString(item.getPrice()));
					data.put("quantity", Integer.toString(item.getQuantity()));
					data.put("short_desc", item.getShortDesc());
					data.put("full_desc", item.getFullDesc());
					data.put("specific_features", item.getSpecificFeatures());
				} catch (NumberFormatException e) {
					CustomException.printException(e, "Cannot convert parameter to integer!");
				} catch (SQLException e) {
					CustomException.printException(e, "SQL error!");
					e.printStackTrace();
				}
				request.setAttribute("data", data);
				
				request.setAttribute("heading", "Editace položky");
				request.getRequestDispatcher("/WEB-INF/admin/item-form.jsp").forward(request, response);
			}else if(lastUriSegment.equals("add")){ /* ITEM ADD */
				List<CategoryBean> categories = null;
				try {
					categories = itemAccess.getCategories();
				} catch (SQLException e) {
					CustomException.printFullException(e, "SQL error!");
					e.printStackTrace();
				}
				request.setAttribute("categories", categories);
				
				request.setAttribute("heading", "Vytvoření nové položky");
				request.getRequestDispatcher("/WEB-INF/admin/item-form.jsp").forward(request, response);
			}
		}else if(pageType.equals("order")){
			if(lastUriSegment.equals("show")){
				List<OrderBean> orderList = null;
				try {
					OrderDao orderAccess =  new OrderDao();
					orderList = orderAccess.getOrders();
				} catch (SQLException e) {
					CustomException.printFullException(e, "SQL error!");
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					CustomException.printException(e, "Cannot found OrderDao class!");
				} catch (DecoderException e) {
					CustomException.printException(e, "Cannot decode serialized data from database!");
				}
				request.setAttribute("orderList", orderList);
				
				request.getRequestDispatcher("/WEB-INF/admin/order-show.jsp").forward(request, response);
			}else if(lastUriSegment.equals("view")){
				request.getRequestDispatcher("/WEB-INF/admin/order-view.jsp").forward(request, response);
			}else if(lastUriSegment.equals("delete")){
				request.getRequestDispatcher("/WEB-INF/admin/order-delete.jsp").forward(request, response);
			}else if(lastUriSegment.equals("edit")){
				request.getRequestDispatcher("/WEB-INF/admin/order-edit.jsp").forward(request, response);
			}else if(lastUriSegment.equals("add")){
				request.getRequestDispatcher("/WEB-INF/admin/order-add.jsp").forward(request, response);
			}
		}
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		boolean error = false;
		boolean edit = false;
		List<String> msg = new ArrayList<String>();
		FormChecker ck = new FormChecker();
		String value;
		
		ItemBean item = null;
		ItemDao itemAccess = null;
		try {
			itemAccess = new ItemDao();
		} catch (ClassNotFoundException e) {
			CustomException.printFullException(e, "Cannot load list of items!");
		}
		
		Map<String, String> form = new HashMap<String,String>();
		String fileName = null;
		InputStream fileContent = null;
		try {
	        List<FileItem> items = new ServletFileUpload(new DiskFileItemFactory()).parseRequest(request);
	        for (FileItem it : items) {
	            if (it.isFormField()) {
	                // Process regular form field (input type="text|radio|checkbox|etc", select, etc).
	            	//String fieldName = item.getFieldName();
	                //String fieldValue = item.getString();
	            	form.put(it.getFieldName(), it.getString());
	            } else {
	                // Process form file field (input type="file").
	                //String fieldName = it.getFieldName();
	                fileName = FilenameUtils.getName(it.getName());
	                fileContent = it.getInputStream();
	            }
	        }
	    } catch (FileUploadException e) {
	        throw new ServletException("Cannot parse multipart request.", e);
	    }

		
		value = form.get("idItem");
		if(value != null && !value.equals("")){
			edit = true;
			try {
				item = itemAccess.read(Integer.valueOf(value));
			} catch (NumberFormatException e) {
				CustomException.printException(e, "Cannot convert parameter to integer!");
			} catch (SQLException e) {
				CustomException.printException(e, "SQL error!");
				e.printStackTrace();
			}
		}else {
			item = new ItemBean();
		}
		
		//add image stream to ItemBean
		if(fileName.length() != 0 && fileName != null){
        	item.setPhoto(fileContent);
        }
		
		value = form.get("name");
		if(value.length() == 0){
			error = true;
			msg.add("Jméno nemůže být prázdné!");
		}else {
			item.setName(value);
		}
		
		value = form.get("price");
		if(value.length() == 0){
			error = true;
			msg.add("Cena nemůže být prázdná!");
		}else if(!ck.isNumber(value) || Integer.parseInt(value) < 0){
			error = true;
			msg.add("Cena musí být kladné číslo!");
		}else {
			item.setPrice(Integer.parseInt(value));
		}
		
		value = form.get("quantity");
		if(!ck.isNumber(value) || Integer.parseInt(value) < 0){
			error = true;
			msg.add("Množství zboží musí být kladné číslo!");
		}else {
			item.setQuantity((Integer.parseInt(value)));
		}
		
		value = form.get("short_desc");
		if(value.length() == 0){
			error = true;
			msg.add("Krátký popis nemůže být prázdný!");
		}else {
			item.setShortDesc(value);
		}
		
		value = form.get("full_desc");
		if(value.length() == 0){
			error = true;
			msg.add("Dlouhý popis nemůže být prázdný!");
		}else {
			item.setFullDesc(value);
		}
		
        value = form.get("specific_features");
		if(value.length() != 0){
			item.setSpecificFeatures(value);
		}
		
		value = form.get("category");
		if(value.length() != 0){
			item.setCategory(value);
		}
        


		
		if(error){
			//control print error messages
			System.err.println("SignUp form error: " + Arrays.toString(msg.toArray()));

			//return original POST data
			Map<String, String> data = new HashMap<String,String>();
			Enumeration<String> parameterNames = request.getParameterNames();
			while (parameterNames.hasMoreElements()) {
				String paramName = parameterNames.nextElement();
				data.put(paramName, request.getParameter(paramName));
			}
			request.setAttribute("data", data);
			
			//setup error
			request.setAttribute("formError", true);
			request.setAttribute("formErrorMsg", msg);
			
			//render sign up form
			this.doGet(request, response);	
		} else {
			try {
				if(edit){
					itemAccess.update(item);
				}else {
					itemAccess.create(item);
				}
			} catch (SQLException e) {
				CustomException.printException(e, "SQL Error");
				e.printStackTrace();
			}
			
			response.sendRedirect("/neumann/admin/item/show");
		}	
	}
}
