package servlet;

import java.io.IOException;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import libs.FormChecker;
import beans.AddressBean;
import beans.UserBean;
import dao.AddressDao;
import dao.UserDao;
import exception.CustomException;

/**
 * Servlet implementation class Profile
 */
public class Profile extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public Profile() {
        super();
    }

	/** 
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		UserBean user = (UserBean) request.getAttribute("user");
		SimpleDateFormat sdf =  new SimpleDateFormat("YYYY-MM-dd");
		
		Map<String, String> data = new HashMap<String,String>();
			data.put("idUser", Integer.toString(user.getIdUser()));
			data.put("name", user.getName());
			data.put("lastname", user.getLastname());
			data.put("email", user.getEmail());
			data.put("date", sdf.format(user.getDateOfBirth().getTime()));
			
		AddressDao accessAddress = null;
		try {
			accessAddress = new AddressDao();
			AddressBean address = accessAddress.getAddress(user);
			//TO-DO: get address by user
			if(address != null){
				data.put("idAddress", Integer.toString(address.getIdAddress()));
				data.put("street", address.getStreet());
				data.put("town", address.getTown());
				data.put("zipCode", Integer.toString(address.getZipCode()));
			}
		} catch (ClassNotFoundException e) {
			CustomException.printException(e, "Cannot find AddressDao class!");
		} catch (SQLException e) {
			CustomException.printFullException(e, "SQL error!");
		}
		
		request.setAttribute("data", data);
		request.getRequestDispatcher("/WEB-INF/user/profile.jsp").forward(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		UserDao accessUser = null;
		AddressDao accessAdr = null;
		
		UserBean user = null;
		AddressBean address = null;
		
		boolean error = false;
		List<String> msg = new ArrayList<String>();
		FormChecker ck = new FormChecker();
		String value;
		
		try {
			accessUser = new UserDao();
			accessAdr = new AddressDao();
			user = accessUser.read(Integer.parseInt(request.getParameter("id_user")));
			address = accessAdr.read(Integer.parseInt(request.getParameter("id_address")));
		} catch (ClassNotFoundException e) {
			CustomException.printException(e, "Cannot find ***Dao class!");
		} catch (NumberFormatException e) {
			CustomException.printException(e, "Cannot parse this String to Integer");
		} catch (SQLException e) {
			CustomException.printException(e, "SQL error!");
		}
		
		
		value = request.getParameter("name");
		if(value.length() == 0){
			error = true;
			msg.add("Jméno nemůže být prázdné!");
		}else {
			user.setName(value);
		}
		
		value = request.getParameter("lastname");
		if(value.length() == 0){
			error = true;
			msg.add("Přijmení nemůže být prázdné!");
		}else {
			user.setLastname(value);
		}
		
		value = request.getParameter("email");
		if(value.length() == 0){
			error = true;
			msg.add("E-mail nemůže být prázdný!");
		}else if(!ck.checkEmail(value)){
			error = true;
			msg.add("Zadejte platný e-mail!");
		}else {
			user.setEmail(value);
		}
		
		value = request.getParameter("date");
		if(value.length() == 0){
			error = true;
			msg.add("Datum narození nemůže být prázdný!");
		}else {
			Calendar birth = Calendar.getInstance();
			SimpleDateFormat sdf = new SimpleDateFormat("YYYY-MM-dd");
			try {
				birth.setTime(sdf.parse(value));
				user.setDateOfBirth(birth);
			} catch (ParseException e) {
				CustomException.printException(e, "Bad date format!");
				error = true;
				msg.add("Datum narození má špatný formát!");
			}
		}
		
		value = request.getParameter("street");
		if(value.length() == 0){
			error = true;
			msg.add("Ulice nemůže být prázdná!");
		}else {
			address.setStreet(value);
		}
		
		value = request.getParameter("town");
		if(value.length() == 0){
			error = true;
			msg.add("Město nemůže být prázdné!");
		}else {
			address.setTown(value);
		}
		
		value = request.getParameter("zipCode");
		if(value.length() == 0){
			error = true;
			msg.add("PSČ nemůže být prázdné!");
		}else if(!ck.isNumber(value) || value.length() != 5){
			error = true;
			msg.add("PSČ musí obsahovat právě 5 číslic!");
		}else {
			address.setZipCode(Integer.parseInt(value));
		}
		
		
		if(error){
			//control print error messages
			System.err.println("Profile form error: " + Arrays.toString(msg.toArray()));

			//return original POST data
			Map<String, String> data = new HashMap<String,String>();
			Enumeration<String> parameterNames = request.getParameterNames();
			while (parameterNames.hasMoreElements()) {
				String paramName = parameterNames.nextElement();
				data.put(paramName, request.getParameter(paramName));
			}
			request.setAttribute("data", data);
			
			//setup error
			request.setAttribute("formError", true);
			request.setAttribute("formErrorMsg", msg);
			
			//render profile form
			this.doGet(request, response);	
		} else {
			try {
				//INSERT user
				accessUser.update(user);
				//INSERT address
				accessAdr.update(address);
			} catch (SQLException e) {
				CustomException.printException(e, "SQL error!");
			}
			
			response.sendRedirect("/neumann/profile");
		}
		
	}

}
