package servlet;

import java.io.IOException;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import libs.FormChecker;

import org.apache.commons.codec.DecoderException;

import beans.AddressBean;
import beans.OrderBean;
import beans.UserBean;
import dao.AddressDao;
import dao.OrderDao;
import exception.CustomException;

/**
 * Servlet implementation class ShoppingCart
 */
public class ShoppingCart extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	/**
	 * For handle errors in form
	 */
	private boolean error;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public ShoppingCart() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Map<String, String> data = new HashMap<String,String>();
		Object isLogged = request.getAttribute("isLogged");
		if(!error && isLogged != null && String.valueOf(isLogged).equalsIgnoreCase("true")){
			UserBean user = (UserBean) request.getAttribute("user");
			SimpleDateFormat sdf =  new SimpleDateFormat("YYYY-MM-dd");
			
			data.put("idUser", Integer.toString(user.getIdUser()));
			data.put("name", user.getName());
			data.put("lastname", user.getLastname());
			data.put("email", user.getEmail());
			data.put("date", sdf.format(user.getDateOfBirth().getTime()));
				
			AddressDao accessAddress = null;
			try {
				accessAddress = new AddressDao();
				AddressBean address = accessAddress.getAddress(user);
				//TO-DO: get address by user
				if(address != null){
					data.put("idAddress", Integer.toString(address.getIdAddress()));
					data.put("street", address.getStreet());
					data.put("town", address.getTown());
					data.put("zipCode", Integer.toString(address.getZipCode()));
				}
			} catch (ClassNotFoundException e) {
				CustomException.printException(e, "Cannot find AddressDao class!");
			} catch (SQLException e) {
				CustomException.printFullException(e, "SQL error!");
			}
			request.setAttribute("data", data);
		}
		
		request.getRequestDispatcher("/WEB-INF/order.jsp").forward(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		boolean billingError = false, deliveryError = false;
		List<String> msg = new ArrayList<String>();
		FormChecker ck = new FormChecker();
		String value;
		
		OrderBean order = new OrderBean();
		
		// set all ordered items 
		String[] items = request.getParameterValues("item");
		order.setOrderedItems(items);
		
		// set total price (before discount)
		value = request.getParameter("total-price");
		order.setTotalPrice((int) Double.parseDouble(value));
		
		// set discount and final price
		value = request.getParameter("discount");
		
		if(value == null){
			order.setDiscount(0);
			order.setFinalPrice(order.getTotalPrice());
		}else {
			order.setDiscount(Integer.parseInt(value));
			double discount = (100 - order.getDiscount());
			int finalPrice = (int) (order.getTotalPrice() * (discount / 100));
			order.setFinalPrice(finalPrice);
		}
		
		// set order date to current time
		Calendar orderDate = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("YYYY-MM-dd");
		try {
			orderDate.setTime(sdf.parse(sdf.format(orderDate.getTime())));
		} catch (ParseException e) {
			CustomException.printException(e, "Cannot parse the date - bad format!");
		}
		order.setOrderDate(orderDate);

		
		/*
		 * Billing address
		 * 
		 */
		value = request.getParameter("name");
		if(value.length() == 0){
			error = billingError = true;
			msg.add("Jméno nemůže být prázdné!");
		}else {
			order.addBillingAddress(value);
		}
		
		value = request.getParameter("lastname");
		if(value.length() == 0){
			error = billingError = true;
			msg.add("Přijmení nemůže být prázdné!");
		}else {
			order.addBillingAddress(value);
		}
		
		value = request.getParameter("email");
		if(value.length() == 0){
			error = billingError = true;
			msg.add("E-mail nemůže být prázdný!");
		}else if(!ck.checkEmail(value)){
			error = true;
			msg.add("Zadejte platný e-mail!");
		}else {
			order.addBillingAddress(value);
		}
		
		value = request.getParameter("street");
		if(value.length() == 0){
			error = billingError = true;
			msg.add("Ulice nemůže být prázdná!");
		}else {
			order.addBillingAddress(value);;
		}
		
		value = request.getParameter("town");
		if(value.length() == 0){
			error = billingError = true;
			msg.add("Město nemůže být prázdné!");
		}else {
			order.addBillingAddress(value);
		}
		
		value = request.getParameter("zipCode");
		if(value.length() == 0){
			error = billingError = true;
			msg.add("PSČ nemůže být prázdné!");
		}else if(!ck.isNumber(value) || value.length() != 5){
			error = true;
			msg.add("PSČ musí obsahovat právě 5 číslic!");
		}else {
			order.addBillingAddress(value);
		}
		
		/* 
		 * Delivery address 
		 * 
		 */
		if(request.getParameter("del-name").length() != 0 || request.getParameter("del-lastname").length() != 0 ||
				request.getParameter("del-street").length() != 0 || request.getParameter("del-town").length() != 0 ||
				request.getParameter("del-zipCode").length() != 0){
		
			value = request.getParameter("del-name");
			if(value.length() == 0){
				error = deliveryError = true;
				msg.add("Jméno nemůže být prázdné!");
			}else {
				order.addShippingAddress(value);
			}
			
			value = request.getParameter("del-lastname");
			if(value.length() == 0){
				error = deliveryError = true;
				msg.add("Přijmení nemůže být prázdné!");
			}else {
				order.addShippingAddress(value);
			}
			
			value = request.getParameter("del-street");
			if(value.length() == 0){
				error = deliveryError = true;
				msg.add("Ulice nemůže být prázdná!");
			}else {
				order.addShippingAddress(value);
			}
			
			value = request.getParameter("del-town");
			if(value.length() == 0){
				error = deliveryError  = true;
				msg.add("Město nemůže být prázdné!");
			}else {
				order.addShippingAddress(value);
			}
			
			value = request.getParameter("del-zipCode");
			if(value.length() == 0){
				error = deliveryError = true;
				msg.add("PSČ nemůže být prázdné!");
			}else if(!ck.isNumber(value) || value.length() != 5){
				error = true;
				msg.add("PSČ musí obsahovat právě 5 číslic!");
			}else {
				order.addShippingAddress(value);
			}
		}
		/* End of delivery address */
		
		value = request.getParameter("agreement");
		if(value == null || !value.equals("true")){
			error = true;
			msg.add("Musíte souhlasit se všeobecnými podmínkami!");
		}
		
		if(error){
			if(deliveryError){
				msg.add(0, "Chyba v doručovací adrese!");
			}else if (billingError){
				msg.add(0, "Chyba ve faktrurační adrese!");
			}
			
			//control print error messages
			System.err.println("SignUp form error: " + Arrays.toString(msg.toArray()));

			//return original POST data
			Map<String, String> data = new HashMap<String,String>();
			Enumeration<String> parameterNames = request.getParameterNames();
			while (parameterNames.hasMoreElements()) {
				String paramName = parameterNames.nextElement();
				data.put(paramName, request.getParameter(paramName));
			}
			request.setAttribute("data", data);
			
			//setup error
			request.setAttribute("formError", true);
			request.setAttribute("formErrorMsg", msg);
			
			//render sign up form
			this.doGet(request, response);	
		} else {
			System.out.println(order.toString());
			
			OrderDao accessOrder = null;
	
			try {
				accessOrder = new OrderDao();
				accessOrder.create(order);
			} catch (ClassNotFoundException e) {
				CustomException.printException(e, "Cannot found OrderDao class!");
			} catch (SQLException e) {
				CustomException.printException(e, "SQL error!");
				e.printStackTrace();
			} catch (DecoderException e) {
				CustomException.printException(e, "Cannot decode serialized data from database!");
			}	

			removeShoppingCart(request);
			response.sendRedirect("/neumann/page/order-finish");
		}
	}
	
	private void removeShoppingCart(HttpServletRequest request){
		HttpSession session = request.getSession(false);
		if(session.getAttribute("shoppingCart") != null){
			session.removeAttribute("shoppingCart");
		}
	}

}
