package servlet;

import java.io.IOException;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import libs.FormChecker;
import beans.AddressBean;
import beans.UserBean;
import dao.AddressDao;
import dao.UserDao;
import exception.CustomException;

/**
 * Servlet implementation class SignUp
 */
public class SignUp extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public SignUp() {
        super();
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String isLogged = String.valueOf(request.getAttribute("isLogged"));
		if(Boolean.valueOf(isLogged)){
			response.sendRedirect("/neumann");
		}else {
			request.getRequestDispatcher("WEB-INF/signup.jsp").forward(request, response);
		}
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		boolean error = false;
		List<String> msg = new ArrayList<String>();
		FormChecker ck = new FormChecker();
		String value;
		
		UserBean user = new UserBean();
		AddressBean address = new AddressBean();
		
		value = request.getParameter("login");
		if(value.length() == 0 || !ck.isAlpaNumeric(value)){
			error = true;
			msg.add("Login musí být vyplněn a smí obsahovat pouze číslice a písmena!");
		}else {
			user.setLogin(value);
		}
		
		value = request.getParameter("password");
		if(value.length() == 0){
			error = true;
			msg.add("Heslo nemůže být prázdné!");
		}else if(!value.equals(request.getParameter("password2"))){
			error = true;
			msg.add("Hesla se musí shodovat!");
		}else if(!ck.isStrongPassword(value)){
			error = true;
			msg.add("Heslo musí být minimálně 5 znaků dlouhé a musí obsahovat písmena i číslice!");
		}else {
			//heslo se šifruje v UserDao
			user.setPassword(value);
		}
		
		value = request.getParameter("name");
		if(value.length() == 0){
			error = true;
			msg.add("Jméno nemůže být prázdné!");
		}else {
			user.setName(value);
		}
		
		value = request.getParameter("lastname");
		if(value.length() == 0){
			error = true;
			msg.add("Přijmení nemůže být prázdné!");
		}else {
			user.setLastname(value);
		}
		
		value = request.getParameter("email");
		if(value.length() == 0){
			error = true;
			msg.add("E-mail nemůže být prázdný!");
		}else if(!ck.checkEmail(value)){
			error = true;
			msg.add("Zadejte platný e-mail!");
		}else {
			user.setEmail(value);
		}
		
		value = request.getParameter("date");
		if(value.length() == 0){
			error = true;
			msg.add("Datum narození nemůže být prázdný!");
		}else {
			Calendar birth = Calendar.getInstance();
			SimpleDateFormat sdf = new SimpleDateFormat("YYYY-MM-dd");
			try {
				birth.setTime(sdf.parse(value));
				user.setDateOfBirth(birth);
			} catch (ParseException e) {
				CustomException.printException(e, "Bad date format!");
				error = true;
				msg.add("Datum narození má špatný formát!");
			}
		}

		
//		value = request.getParameter("phone");
//		if(value.length() != 0 && !ck.checkPhone(value)){
//			error = true;
//			msg.add("Telefonní číslo musí obsahovat právě 9 číslic!");
//		}
		
		value = request.getParameter("street");
		if(value.length() == 0){
			error = true;
			msg.add("Ulice nemůže být prázdná!");
		}else {
			address.setStreet(value);
		}
		
		value = request.getParameter("town");
		if(value.length() == 0){
			error = true;
			msg.add("Město nemůže být prázdné!");
		}else {
			address.setTown(value);
		}
		
		value = request.getParameter("zipCode");
		if(value.length() == 0){
			error = true;
			msg.add("PSČ nemůže být prázdné!");
		}else if(!ck.isNumber(value) || value.length() != 5){
			error = true;
			msg.add("PSČ musí obsahovat právě 5 číslic!");
		}else {
			address.setZipCode(Integer.parseInt(value));
		}
		
//		value = request.getParameter("identNum");
//		if(value.length() != 0 && (!ck.isNumber(value) || value.length() != 8)){
//			error = true;
//			msg.add("IČ musí obsahovat právě 8 číslic!");
//		}
		
		//DIČ se neověřuje
		
//		value = request.getParameter("sex");
//		if(value.equals("0")){
//			error = true;
//			msg.add("Vyberte prosím nějaké pohlaví!");
//		}
		
		value = request.getParameter("agreement");
		if(value == null || !value.equals("true")){
			error = true;
			msg.add("Musíte souhlasit se všeobecnými podmínkami!");
		}
		
		value = request.getParameter("captcha");
		if(value == null || !value.equals("5")){
			error = true;
			msg.add("Musíte vyplnit ochranu proti robotům!");
		}
		
		
		
		if(error){
			//control print error messages
			System.err.println("SignUp form error: " + Arrays.toString(msg.toArray()));

			//return original POST data
			Map<String, String> data = new HashMap<String,String>();
			Enumeration<String> parameterNames = request.getParameterNames();
			while (parameterNames.hasMoreElements()) {
				String paramName = parameterNames.nextElement();
				data.put(paramName, request.getParameter(paramName));
			}
			request.setAttribute("data", data);
			
			//setup error
			request.setAttribute("formError", true);
			request.setAttribute("formErrorMsg", msg);
			
			//render sign up form
			this.doGet(request, response);	
		} else {
			List<AddressBean> adr = new ArrayList<AddressBean>();
			adr.add(address);
			user.setAddresses(adr);
			
			UserDao accessUser = null;
			AddressDao accessAdr = null;
			try {
				//INSERT user
				accessUser = new UserDao();
				accessUser.create(user);
				//INSERT address
				accessAdr = new AddressDao();
				accessAdr.create(address);
			} catch (ClassNotFoundException e) {
				CustomException.printException(e, "Cannot find UserDao class!");
			} catch (SQLException e) {
				CustomException.printException(e, "SQL error!");
			}
			
			response.sendRedirect("/neumann/login?signup=success");
		}
	}

}
